# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common


class UnformattedMessageListType(GeneratedsSuper):
    """The UnformattedMessageListType type is a list of unformatted
    messages in the event log entry."""

    subclass = None
    superclass = None
    def __init__(self, Unformatted_Message=None):
        if Unformatted_Message is None:
            self.Unformatted_Message = []
        else:
            self.Unformatted_Message = Unformatted_Message
    def factory(*args_, **kwargs_):
        if UnformattedMessageListType.subclass:
            return UnformattedMessageListType.subclass(*args_, **kwargs_)
        else:
            return UnformattedMessageListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Unformatted_Message(self): return self.Unformatted_Message
    def set_Unformatted_Message(self, Unformatted_Message): self.Unformatted_Message = Unformatted_Message
    def add_Unformatted_Message(self, value): self.Unformatted_Message.append(value)
    def insert_Unformatted_Message(self, index, value): self.Unformatted_Message[index] = value
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Unformatted_Message
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinEventLogObj:', name_='UnformattedMessageListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='UnformattedMessageListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinEventLogObj:', name_='UnformattedMessageListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinEventLogObj:', name_='UnformattedMessageListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Unformatted_Message_ in self.Unformatted_Message:
            Unformatted_Message_.export(lwrite, level, 'WinEventLogObj:', name_='Unformatted_Message', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Unformatted_Message':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.Unformatted_Message.append(obj_)
# end class UnformattedMessageListType

class WindowsEventLogObjectType(cybox_common.ObjectPropertiesType):
    """The WindowsEventLogObjectType type is intended to characterize
    entries in the Windows event log."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, EID=None, Type=None, Log=None, Message=None, Category_Num=None, Category=None, Generation_Time=None, Source=None, Machine=None, User=None, Blob=None, Correlation_Activity_ID=None, Correlation_Related_Activity_ID=None, Execution_Process_ID=None, Execution_Thread_ID=None, Index=None, Reserved=None, Unformatted_Message_List=None, Write_Time=None):
        super(WindowsEventLogObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.EID = EID
        self.Type = Type
        self.Log = Log
        self.Message = Message
        self.Category_Num = Category_Num
        self.Category = Category
        self.Generation_Time = Generation_Time
        self.Source = Source
        self.Machine = Machine
        self.User = User
        self.Blob = Blob
        self.Correlation_Activity_ID = Correlation_Activity_ID
        self.Correlation_Related_Activity_ID = Correlation_Related_Activity_ID
        self.Execution_Process_ID = Execution_Process_ID
        self.Execution_Thread_ID = Execution_Thread_ID
        self.Index = Index
        self.Reserved = Reserved
        self.Unformatted_Message_List = Unformatted_Message_List
        self.Write_Time = Write_Time
    def factory(*args_, **kwargs_):
        if WindowsEventLogObjectType.subclass:
            return WindowsEventLogObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsEventLogObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_EID(self): return self.EID
    def set_EID(self, EID): self.EID = EID
    def validate_LongObjectPropertyType(self, value):
        # Validate type cybox_common.LongObjectPropertyType, a restriction on None.
        pass
    def get_Type(self): return self.Type
    def set_Type(self, Type): self.Type = Type
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Log(self): return self.Log
    def set_Log(self, Log): self.Log = Log
    def get_Message(self): return self.Message
    def set_Message(self, Message): self.Message = Message
    def get_Category_Num(self): return self.Category_Num
    def set_Category_Num(self, Category_Num): self.Category_Num = Category_Num
    def get_Category(self): return self.Category
    def set_Category(self, Category): self.Category = Category
    def get_Generation_Time(self): return self.Generation_Time
    def set_Generation_Time(self, Generation_Time): self.Generation_Time = Generation_Time
    def validate_DateTimeObjectPropertyType(self, value):
        # Validate type cybox_common.DateTimeObjectPropertyType, a restriction on None.
        pass
    def get_Source(self): return self.Source
    def set_Source(self, Source): self.Source = Source
    def get_Machine(self): return self.Machine
    def set_Machine(self, Machine): self.Machine = Machine
    def get_User(self): return self.User
    def set_User(self, User): self.User = User
    def get_Blob(self): return self.Blob
    def set_Blob(self, Blob): self.Blob = Blob
    def validate_Base64BinaryObjectPropertyType(self, value):
        # Validate type cybox_common.Base64BinaryObjectPropertyType, a restriction on None.
        pass
    def get_Correlation_Activity_ID(self): return self.Correlation_Activity_ID
    def set_Correlation_Activity_ID(self, Correlation_Activity_ID): self.Correlation_Activity_ID = Correlation_Activity_ID
    def get_Correlation_Related_Activity_ID(self): return self.Correlation_Related_Activity_ID
    def set_Correlation_Related_Activity_ID(self, Correlation_Related_Activity_ID): self.Correlation_Related_Activity_ID = Correlation_Related_Activity_ID
    def get_Execution_Process_ID(self): return self.Execution_Process_ID
    def set_Execution_Process_ID(self, Execution_Process_ID): self.Execution_Process_ID = Execution_Process_ID
    def get_Execution_Thread_ID(self): return self.Execution_Thread_ID
    def set_Execution_Thread_ID(self, Execution_Thread_ID): self.Execution_Thread_ID = Execution_Thread_ID
    def get_Index(self): return self.Index
    def set_Index(self, Index): self.Index = Index
    def get_Reserved(self): return self.Reserved
    def set_Reserved(self, Reserved): self.Reserved = Reserved
    def get_Unformatted_Message_List(self): return self.Unformatted_Message_List
    def set_Unformatted_Message_List(self, Unformatted_Message_List): self.Unformatted_Message_List = Unformatted_Message_List
    def get_Write_Time(self): return self.Write_Time
    def set_Write_Time(self, Write_Time): self.Write_Time = Write_Time
    def hasContent_(self):
        if (
            self.EID is not None or
            self.Type is not None or
            self.Log is not None or
            self.Message is not None or
            self.Category_Num is not None or
            self.Category is not None or
            self.Generation_Time is not None or
            self.Source is not None or
            self.Machine is not None or
            self.User is not None or
            self.Blob is not None or
            self.Correlation_Activity_ID is not None or
            self.Correlation_Related_Activity_ID is not None or
            self.Execution_Process_ID is not None or
            self.Execution_Thread_ID is not None or
            self.Index is not None or
            self.Reserved is not None or
            self.Unformatted_Message_List is not None or
            self.Write_Time is not None or
            super(WindowsEventLogObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinEventLogObj:', name_='WindowsEventLogObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsEventLogObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinEventLogObj:', name_='WindowsEventLogObjectType'):
        super(WindowsEventLogObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsEventLogObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinEventLogObj:', name_='WindowsEventLogObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsEventLogObjectType, self).exportChildren(lwrite, level, 'WinEventLogObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.EID is not None:
            self.EID.export(lwrite, level, 'WinEventLogObj:', name_='EID', pretty_print=pretty_print)
        if self.Type is not None:
            self.Type.export(lwrite, level, 'WinEventLogObj:', name_='Type', pretty_print=pretty_print)
        if self.Log is not None:
            self.Log.export(lwrite, level, 'WinEventLogObj:', name_='Log', pretty_print=pretty_print)
        if self.Message is not None:
            self.Message.export(lwrite, level, 'WinEventLogObj:', name_='Message', pretty_print=pretty_print)
        if self.Category_Num is not None:
            self.Category_Num.export(lwrite, level, 'WinEventLogObj:', name_='Category_Num', pretty_print=pretty_print)
        if self.Category is not None:
            self.Category.export(lwrite, level, 'WinEventLogObj:', name_='Category', pretty_print=pretty_print)
        if self.Generation_Time is not None:
            self.Generation_Time.export(lwrite, level, 'WinEventLogObj:', name_='Generation_Time', pretty_print=pretty_print)
        if self.Source is not None:
            self.Source.export(lwrite, level, 'WinEventLogObj:', name_='Source', pretty_print=pretty_print)
        if self.Machine is not None:
            self.Machine.export(lwrite, level, 'WinEventLogObj:', name_='Machine', pretty_print=pretty_print)
        if self.User is not None:
            self.User.export(lwrite, level, 'WinEventLogObj:', name_='User', pretty_print=pretty_print)
        if self.Blob is not None:
            self.Blob.export(lwrite, level, 'WinEventLogObj:', name_='Blob', pretty_print=pretty_print)
        if self.Correlation_Activity_ID is not None:
            self.Correlation_Activity_ID.export(lwrite, level, 'WinEventLogObj:', name_='Correlation_Activity_ID', pretty_print=pretty_print)
        if self.Correlation_Related_Activity_ID is not None:
            self.Correlation_Related_Activity_ID.export(lwrite, level, 'WinEventLogObj:', name_='Correlation_Related_Activity_ID', pretty_print=pretty_print)
        if self.Execution_Process_ID is not None:
            self.Execution_Process_ID.export(lwrite, level, 'WinEventLogObj:', name_='Execution_Process_ID', pretty_print=pretty_print)
        if self.Execution_Thread_ID is not None:
            self.Execution_Thread_ID.export(lwrite, level, 'WinEventLogObj:', name_='Execution_Thread_ID', pretty_print=pretty_print)
        if self.Index is not None:
            self.Index.export(lwrite, level, 'WinEventLogObj:', name_='Index', pretty_print=pretty_print)
        if self.Reserved is not None:
            self.Reserved.export(lwrite, level, 'WinEventLogObj:', name_='Reserved', pretty_print=pretty_print)
        if self.Unformatted_Message_List is not None:
            self.Unformatted_Message_List.export(lwrite, level, 'WinEventLogObj:', name_='Unformatted_Message_List', pretty_print=pretty_print)
        if self.Write_Time is not None:
            self.Write_Time.export(lwrite, level, 'WinEventLogObj:', name_='Write_Time', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsEventLogObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'EID':
            obj_ = cybox_common.LongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_EID(obj_)
        elif nodeName_ == 'Type':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Type(obj_)
        elif nodeName_ == 'Log':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Log(obj_)
        elif nodeName_ == 'Message':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Message(obj_)
        elif nodeName_ == 'Category_Num':
            obj_ = cybox_common.LongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Category_Num(obj_)
        elif nodeName_ == 'Category':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Category(obj_)
        elif nodeName_ == 'Generation_Time':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Generation_Time(obj_)
        elif nodeName_ == 'Source':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Source(obj_)
        elif nodeName_ == 'Machine':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Machine(obj_)
        elif nodeName_ == 'User':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_User(obj_)
        elif nodeName_ == 'Blob':
            obj_ = cybox_common.Base64BinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Blob(obj_)
        elif nodeName_ == 'Correlation_Activity_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Correlation_Activity_ID(obj_)
        elif nodeName_ == 'Correlation_Related_Activity_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Correlation_Related_Activity_ID(obj_)
        elif nodeName_ == 'Execution_Process_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Execution_Process_ID(obj_)
        elif nodeName_ == 'Execution_Thread_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Execution_Thread_ID(obj_)
        elif nodeName_ == 'Index':
            obj_ = cybox_common.LongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Index(obj_)
        elif nodeName_ == 'Reserved':
            obj_ = cybox_common.LongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Reserved(obj_)
        elif nodeName_ == 'Unformatted_Message_List':
            obj_ = UnformattedMessageListType.factory()
            obj_.build(child_)
            self.set_Unformatted_Message_List(obj_)
        elif nodeName_ == 'Write_Time':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Write_Time(obj_)
        super(WindowsEventLogObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsEventLogObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Correlation_Related_Activity_ID': cybox_common.StringObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Write_Time': cybox_common.DateTimeObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Execution_Thread_ID': cybox_common.StringObjectPropertyType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Unformatted_Message': cybox_common.StringObjectPropertyType,
    'Index': cybox_common.LongObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Source': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Generation_Time': cybox_common.DateTimeObjectPropertyType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Correlation_Activity_ID': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Category': cybox_common.StringObjectPropertyType,
    'Log': cybox_common.StringObjectPropertyType,
    'Category_Num': cybox_common.LongObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'User': cybox_common.StringObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Libraries': cybox_common.LibrariesType,
    'Contributors': cybox_common.PersonnelType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Blob': cybox_common.Base64BinaryObjectPropertyType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Machine': cybox_common.StringObjectPropertyType,
    'EID': cybox_common.LongObjectPropertyType,
    'Execution_Process_ID': cybox_common.StringObjectPropertyType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Message': cybox_common.StringObjectPropertyType,
    'Reserved': cybox_common.LongObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Event_Log'
        rootClass = WindowsEventLogObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Event_Log'
        rootClass = WindowsEventLogObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Event_Log'
        rootClass = WindowsEventLogObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_Event_Log",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsEventLogObjectType",
    "UnformattedMessageListType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.common import ObjectProperties, Property
from cybox.common.object_properties import ObjectPropertiesFactory
from cybox.objects.address_object import Address
from cybox.test import EntityTestCase


class TestProperty(EntityTestCase, unittest.TestCase):
    klass = Property

    _full_dict = {
        'id': "example:Property-1",
        'name': "FilePurpose",
        'description': "The purpose of the file",
        'value': u("Certificate"),
    }


class TestObjectProperties(EntityTestCase, unittest.TestCase):
    klass = Address

    _full_dict = {
        'object_reference': "cybox:address-1",
        'category': Address.CAT_IPV4,
        'xsi:type': Address._XSI_TYPE,
    }

    def test_empty_dict(self):
        self.assertEqual(None, ObjectPropertiesFactory.from_dict({}))

    def test_no_xsi_type(self):
        # d does not have the required 'xsi:type' key.
        d = {'key': 'value'}

        self.assertRaises(ValueError, ObjectPropertiesFactory.from_dict, d)

    def test_detect_address(self):
        d = {'xsi:type': Address._XSI_TYPE}

        obj = ObjectPropertiesFactory.from_dict(d)

        self.assertTrue(isinstance(obj, ObjectProperties))
        self.assertTrue(isinstance(obj, Address))


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_hook_object as win_hook_binding
from cybox.objects.library_object import Library
from cybox.objects.win_handle_object import WinHandle
from cybox.common import ObjectProperties, String, NonNegativeInteger


class WinHook(ObjectProperties):
    _binding = win_hook_binding
    _binding_class = win_hook_binding.WindowsHookObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-hook-1'
    _XSI_NS = "WinHookObj"
    _XSI_TYPE = "WindowsHookObjectType"

    type_ = fields.TypedField("Type", String)
    handle = fields.TypedField("Handle", WinHandle)
    hooking_function_name = fields.TypedField("Hooking_Function_Name", String)
    hooking_module = fields.TypedField("Hooking_Module", Library)
    thread_id = fields.TypedField("Thread_ID", NonNegativeInteger)



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import os
import sys

import cybox.bindings.cybox_core as core_binding
from cybox.core import Observables


def from_file(filename):
    cybox_obj = core_binding.parse(os.path.abspath(filename))
    return Observables.from_obj(cybox_obj)


def main():
    if len(sys.argv) < 2:
        print("Argument required")
        return

    print(from_file(sys.argv[1]).to_json())

if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Tests for various encoding issues throughout the library"""

import unittest

from mixbox.vendor.six import StringIO


class CIQAddressTests(unittest.TestCase):

    def test_can_load_ciq_extension(self):
        from cybox.bindings.extensions.location import ciq_address_3_0
        addr = ciq_address_3_0.CIQAddress3_0InstanceType()

        # Really basic test to verify the extension works.
        s = StringIO()
        addr.export(s.write, 0)
        xml = s.getvalue()
        self.assertEqual(173, len(xml))


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import address_object


class AttachmentsType(GeneratedsSuper):
    """The AttachmenstType captures a list of attachments for an email
    message."""

    subclass = None
    superclass = None
    def __init__(self, File=None):
        if File is None:
            self.File = []
        else:
            self.File = File
    def factory(*args_, **kwargs_):
        if AttachmentsType.subclass:
            return AttachmentsType.subclass(*args_, **kwargs_)
        else:
            return AttachmentsType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_File(self): return self.File
    def set_File(self, File): self.File = File
    def add_File(self, value): self.File.append(value)
    def insert_File(self, index, value): self.File[index] = value
    def hasContent_(self):
        if (
            self.File
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='AttachmentsType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='AttachmentsType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='AttachmentsType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='AttachmentsType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for File_ in self.File:
            File_.export(lwrite, level, 'EmailMessageObj:', name_='File', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'File':
            obj_ = AttachmentReferenceType.factory()
            obj_.build(child_)
            self.File.append(obj_)
# end class AttachmentsType

class EmailHeaderType(GeneratedsSuper):
    """The EmailHeaderType captures a representation of a standard email
    header."""

    subclass = None
    superclass = None
    def __init__(self, Received_Lines=None, To=None, CC=None, BCC=None, From=None, Subject=None, In_Reply_To=None, Date=None, Message_ID=None, Sender=None, Reply_To=None, Errors_To=None, Boundary=None, Content_Type=None, MIME_Version=None, Precedence=None, User_Agent=None, X_Mailer=None, X_Originating_IP=None, X_Priority=None):
        self.Received_Lines = Received_Lines
        self.To = To
        self.CC = CC
        self.BCC = BCC
        self.From = From
        self.Subject = Subject
        self.In_Reply_To = In_Reply_To
        self.Date = Date
        self.Message_ID = Message_ID
        self.Sender = Sender
        self.Reply_To = Reply_To
        self.Errors_To = Errors_To
        self.Boundary = Boundary
        self.Content_Type = Content_Type
        self.MIME_Version = MIME_Version
        self.Precedence = Precedence
        self.User_Agent = User_Agent
        self.X_Mailer = X_Mailer
        self.X_Originating_IP = X_Originating_IP
        self.X_Priority = X_Priority
    def factory(*args_, **kwargs_):
        if EmailHeaderType.subclass:
            return EmailHeaderType.subclass(*args_, **kwargs_)
        else:
            return EmailHeaderType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Received_Lines(self): return self.Received_Lines
    def set_Received_Lines(self, Received_Lines): self.Received_Lines = Received_Lines
    def get_To(self): return self.To
    def set_To(self, To): self.To = To
    def get_CC(self): return self.CC
    def set_CC(self, CC): self.CC = CC
    def get_BCC(self): return self.BCC
    def set_BCC(self, BCC): self.BCC = BCC
    def get_From(self): return self.From
    def set_From(self, From): self.From = From
    def get_Subject(self): return self.Subject
    def set_Subject(self, Subject): self.Subject = Subject
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_In_Reply_To(self): return self.In_Reply_To
    def set_In_Reply_To(self, In_Reply_To): self.In_Reply_To = In_Reply_To
    def get_Date(self): return self.Date
    def set_Date(self, Date): self.Date = Date
    def validate_DateTimeObjectPropertyType(self, value):
        # Validate type cybox_common.DateTimeObjectPropertyType, a restriction on None.
        pass
    def get_Message_ID(self): return self.Message_ID
    def set_Message_ID(self, Message_ID): self.Message_ID = Message_ID
    def get_Sender(self): return self.Sender
    def set_Sender(self, Sender): self.Sender = Sender
    def get_Reply_To(self): return self.Reply_To
    def set_Reply_To(self, Reply_To): self.Reply_To = Reply_To
    def get_Errors_To(self): return self.Errors_To
    def set_Errors_To(self, Errors_To): self.Errors_To = Errors_To
    def get_Boundary(self): return self.Boundary
    def set_Boundary(self, Boundary): self.Boundary = Boundary
    def get_Content_Type(self): return self.Content_Type
    def set_Content_Type(self, Content_Type): self.Content_Type = Content_Type
    def get_MIME_Version(self): return self.MIME_Version
    def set_MIME_Version(self, MIME_Version): self.MIME_Version = MIME_Version
    def get_Precedence(self): return self.Precedence
    def set_Precedence(self, Precedence): self.Precedence = Precedence
    def get_User_Agent(self): return self.User_Agent
    def set_User_Agent(self, User_Agent): self.User_Agent = User_Agent
    def get_X_Mailer(self): return self.X_Mailer
    def set_X_Mailer(self, X_Mailer): self.X_Mailer = X_Mailer
    def get_X_Originating_IP(self): return self.X_Originating_IP
    def set_X_Originating_IP(self, X_Originating_IP): self.X_Originating_IP = X_Originating_IP
    def get_X_Priority(self): return self.X_Priority
    def set_X_Priority(self, X_Priority): self.X_Priority = X_Priority
    def validate_PositiveIntegerObjectPropertyType(self, value):
        # Validate type cybox_common.PositiveIntegerObjectPropertyType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Received_Lines is not None or
            self.To is not None or
            self.CC is not None or
            self.BCC is not None or
            self.From is not None or
            self.Subject is not None or
            self.In_Reply_To is not None or
            self.Date is not None or
            self.Message_ID is not None or
            self.Sender is not None or
            self.Reply_To is not None or
            self.Errors_To is not None or
            self.Boundary is not None or
            self.Content_Type is not None or
            self.MIME_Version is not None or
            self.Precedence is not None or
            self.User_Agent is not None or
            self.X_Mailer is not None or
            self.X_Originating_IP is not None or
            self.X_Priority is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailHeaderType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailHeaderType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='EmailHeaderType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailHeaderType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Received_Lines is not None:
            self.Received_Lines.export(lwrite, level, 'EmailMessageObj:', name_='Received_Lines', pretty_print=pretty_print)
        if self.To is not None:
            self.To.export(lwrite, level, 'EmailMessageObj:', name_='To', pretty_print=pretty_print)
        if self.CC is not None:
            self.CC.export(lwrite, level, 'EmailMessageObj:', name_='CC', pretty_print=pretty_print)
        if self.BCC is not None:
            self.BCC.export(lwrite, level, 'EmailMessageObj:', name_='BCC', pretty_print=pretty_print)
        if self.From is not None:
            self.From.export(lwrite, level, 'EmailMessageObj:', name_='From', pretty_print=pretty_print)
        if self.Subject is not None:
            self.Subject.export(lwrite, level, 'EmailMessageObj:', name_='Subject', pretty_print=pretty_print)
        if self.In_Reply_To is not None:
            self.In_Reply_To.export(lwrite, level, 'EmailMessageObj:', name_='In_Reply_To', pretty_print=pretty_print)
        if self.Date is not None:
            self.Date.export(lwrite, level, 'EmailMessageObj:', name_='Date', pretty_print=pretty_print)
        if self.Message_ID is not None:
            self.Message_ID.export(lwrite, level, 'EmailMessageObj:', name_='Message_ID', pretty_print=pretty_print)
        if self.Sender is not None:
            self.Sender.export(lwrite, level, 'EmailMessageObj:', name_='Sender', pretty_print=pretty_print)
        if self.Reply_To is not None:
            self.Reply_To.export(lwrite, level, 'EmailMessageObj:', name_='Reply_To', pretty_print=pretty_print)
        if self.Errors_To is not None:
            self.Errors_To.export(lwrite, level, 'EmailMessageObj:', name_='Errors_To', pretty_print=pretty_print)
        if self.Boundary is not None:
            self.Boundary.export(lwrite, level, 'EmailMessageObj:', name_='Boundary', pretty_print=pretty_print)
        if self.Content_Type is not None:
            self.Content_Type.export(lwrite, level, 'EmailMessageObj:', name_='Content_Type', pretty_print=pretty_print)
        if self.MIME_Version is not None:
            self.MIME_Version.export(lwrite, level, 'EmailMessageObj:', name_='MIME_Version', pretty_print=pretty_print)
        if self.Precedence is not None:
            self.Precedence.export(lwrite, level, 'EmailMessageObj:', name_='Precedence', pretty_print=pretty_print)
        if self.User_Agent is not None:
            self.User_Agent.export(lwrite, level, 'EmailMessageObj:', name_='User_Agent', pretty_print=pretty_print)
        if self.X_Mailer is not None:
            self.X_Mailer.export(lwrite, level, 'EmailMessageObj:', name_='X_Mailer', pretty_print=pretty_print)
        if self.X_Originating_IP is not None:
            self.X_Originating_IP.export(lwrite, level, 'EmailMessageObj:', name_='X_Originating_IP', pretty_print=pretty_print)
        if self.X_Priority is not None:
            self.X_Priority.export(lwrite, level, 'EmailMessageObj:', name_='X_Priority', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Received_Lines':
            obj_ = EmailReceivedLineListType.factory()
            obj_.build(child_)
            self.set_Received_Lines(obj_)
        elif nodeName_ == 'To':
            obj_ = EmailRecipientsType.factory()
            obj_.build(child_)
            self.set_To(obj_)
        elif nodeName_ == 'CC':
            obj_ = EmailRecipientsType.factory()
            obj_.build(child_)
            self.set_CC(obj_)
        elif nodeName_ == 'BCC':
            obj_ = EmailRecipientsType.factory()
            obj_.build(child_)
            self.set_BCC(obj_)
        elif nodeName_ == 'From':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.set_From(obj_)
        elif nodeName_ == 'Subject':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Subject(obj_)
        elif nodeName_ == 'In_Reply_To':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_In_Reply_To(obj_)
        elif nodeName_ == 'Date':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Date(obj_)
        elif nodeName_ == 'Message_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Message_ID(obj_)
        elif nodeName_ == 'Sender':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.set_Sender(obj_)
        elif nodeName_ == 'Reply_To':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.set_Reply_To(obj_)
        elif nodeName_ == 'Errors_To':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Errors_To(obj_)
        elif nodeName_ == 'Boundary':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Boundary(obj_)
        elif nodeName_ == 'Content_Type':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Content_Type(obj_)
        elif nodeName_ == 'MIME_Version':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_MIME_Version(obj_)
        elif nodeName_ == 'Precedence':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Precedence(obj_)
        elif nodeName_ == 'User_Agent':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_User_Agent(obj_)
        elif nodeName_ == 'X_Mailer':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_X_Mailer(obj_)
        elif nodeName_ == 'X_Originating_IP':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.set_X_Originating_IP(obj_)
        elif nodeName_ == 'X_Priority':
            obj_ = cybox_common.PositiveIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_X_Priority(obj_)
# end class EmailHeaderType

class EmailRecipientsType(GeneratedsSuper):
    """The EmailRecipientsType captures a list of recipients for an email
    message."""

    subclass = None
    superclass = None
    def __init__(self, Recipient=None):
        if Recipient is None:
            self.Recipient = []
        else:
            self.Recipient = Recipient
    def factory(*args_, **kwargs_):
        if EmailRecipientsType.subclass:
            return EmailRecipientsType.subclass(*args_, **kwargs_)
        else:
            return EmailRecipientsType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Recipient(self): return self.Recipient
    def set_Recipient(self, Recipient): self.Recipient = Recipient
    def add_Recipient(self, value): self.Recipient.append(value)
    def insert_Recipient(self, index, value): self.Recipient[index] = value
    def hasContent_(self):
        if (
            self.Recipient
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailRecipientsType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailRecipientsType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='EmailRecipientsType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailRecipientsType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Recipient_ in self.Recipient:
            Recipient_.export(lwrite, level, 'EmailMessageObj:', name_='Recipient', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Recipient':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.Recipient.append(obj_)
# end class EmailRecipientsType

class LinksType(GeneratedsSuper):
    """The LinksType captures a list of URIs, representing the links
    contained in the message."""

    subclass = None
    superclass = None
    def __init__(self, Link=None):
        if Link is None:
            self.Link = []
        else:
            self.Link = Link
    def factory(*args_, **kwargs_):
        if LinksType.subclass:
            return LinksType.subclass(*args_, **kwargs_)
        else:
            return LinksType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Link(self): return self.Link
    def set_Link(self, Link): self.Link = Link
    def add_Link(self, value): self.Link.append(value)
    def insert_Link(self, index, value): self.Link[index] = value
    def hasContent_(self):
        if (
            self.Link
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='LinksType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='LinksType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='LinksType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='LinksType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Link_ in self.Link:
            Link_.export(lwrite, level, 'EmailMessageObj:', name_='Link', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Link':
            obj_ = LinkReferenceType.factory()
            obj_.build(child_)
            self.Link.append(obj_)
# end class LinksType

class EmailReceivedLineType(GeneratedsSuper):
    """The EmailReceivedLineType captures a single 'Received' line in an
    email message header."""
    subclass = None
    superclass = None
    def __init__(self, From=None, By=None, Via=None, With=None, For=None, ID=None, Timestamp=None):
        self.From = From
        self.By = By
        self.Via = Via
        self.With = With
        self.For = For
        self.ID = ID
        self.Timestamp = Timestamp
    def factory(*args_, **kwargs_):
        if EmailReceivedLineType.subclass:
            return EmailReceivedLineType.subclass(*args_, **kwargs_)
        else:
            return EmailReceivedLineType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_From(self): return self.From
    def set_From(self, From): self.From = From
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_By(self): return self.By
    def set_By(self, By): self.By = By
    def get_Via(self): return self.Via
    def set_Via(self, Via): self.Via = Via
    def get_With(self): return self.With
    def set_With(self, With): self.With = With
    def get_For(self): return self.For
    def set_For(self, For): self.For = For
    def get_ID(self): return self.ID
    def set_ID(self, ID): self.ID = ID
    def get_Timestamp(self): return self.Timestamp
    def set_Timestamp(self, Timestamp): self.Timestamp = Timestamp
    def hasContent_(self):
        if (
            self.From is not None or
            self.By is not None or
            self.Via is not None or
            self.With is not None or
            self.For is not None or
            self.ID is not None or
            self.Timestamp is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailReceivedLineType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailReceivedLineType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='EmailReceivedLineType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailReceivedLineType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.From is not None:
            self.From.export(lwrite, level, 'EmailMessageObj:', name_='From', pretty_print=pretty_print)
        if self.By is not None:
            self.By.export(lwrite, level, 'EmailMessageObj:', name_='By', pretty_print=pretty_print)
        if self.Via is not None:
            self.Via.export(lwrite, level, 'EmailMessageObj:', name_='Via', pretty_print=pretty_print)
        if self.With is not None:
            self.With.export(lwrite, level, 'EmailMessageObj:', name_='With', pretty_print=pretty_print)
        if self.For is not None:
            self.For.export(lwrite, level, 'EmailMessageObj:', name_='For', pretty_print=pretty_print)
        if self.ID is not None:
            self.ID.export(lwrite, level, 'EmailMessageObj:', name_='ID', pretty_print=pretty_print)
        if self.Timestamp is not None:
            self.Timestamp.export(lwrite, level, 'EmailMessageObj:', name_='Timestamp', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'From':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_From(obj_)
        elif nodeName_ == 'By':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_By(obj_)
        elif nodeName_ == 'Via':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Via(obj_)
        elif nodeName_ == 'With':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_With(obj_)
        elif nodeName_ == 'For':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_For(obj_)
        elif nodeName_ == 'ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_ID(obj_)
        elif nodeName_ == 'Timestamp':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Timestamp(obj_)
# end class EmailReceivedLineType

class EmailReceivedLineListType(GeneratedsSuper):
    """The EmailReceivedLineListType captures a list of 'Received' lines in
    an email message header."""

    subclass = None
    superclass = None
    def __init__(self, Received=None):
        if Received is None:
            self.Received = []
        else:
            self.Received = Received
    def factory(*args_, **kwargs_):
        if EmailReceivedLineListType.subclass:
            return EmailReceivedLineListType.subclass(*args_, **kwargs_)
        else:
            return EmailReceivedLineListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Received(self): return self.Received
    def set_Received(self, Received): self.Received = Received
    def add_Received(self, value): self.Received.append(value)
    def insert_Received(self, index, value): self.Received[index] = value
    def hasContent_(self):
        if (
            self.Received
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailReceivedLineListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailReceivedLineListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='EmailReceivedLineListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailReceivedLineListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Received_ in self.Received:
            Received_.export(lwrite, level, 'EmailMessageObj:', name_='Received', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Received':
            obj_ = EmailReceivedLineType.factory()
            obj_.build(child_)
            self.Received.append(obj_)
# end class EmailReceivedLineListType

class AttachmentReferenceType(GeneratedsSuper):
    """The AttachmentReferenceType specifies a reference to an Object
    defined elsewhere in the document which characterizes an
    attachment included in the email message.The object_reference
    field specifies a reference to an file-oriented (i.e., the File
    Object or one its derivations such as the Windows File Object)
    Object defined elsewhere in the document, via its id."""

    subclass = None
    superclass = None
    def __init__(self, object_reference=None):
        self.object_reference = _cast(None, object_reference)
        pass
    def factory(*args_, **kwargs_):
        if AttachmentReferenceType.subclass:
            return AttachmentReferenceType.subclass(*args_, **kwargs_)
        else:
            return AttachmentReferenceType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_object_reference(self): return self.object_reference
    def set_object_reference(self, object_reference): self.object_reference = object_reference
    def hasContent_(self):
        if (

            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='AttachmentReferenceType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='AttachmentReferenceType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='AttachmentReferenceType'):
        if self.object_reference is not None:

            lwrite(' object_reference=%s' % (quote_attrib(self.object_reference), ))
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='AttachmentReferenceType', fromsubclass_=False, pretty_print=True):
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('object_reference', node)
        if value is not None:

            self.object_reference = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class AttachmentReferenceType

class LinkReferenceType(GeneratedsSuper):
    """The LinkReferenceType specifies a reference to a URI Object defined
    elsewhere in the document which characterizes a hyperlink
    embedded in the body of the email message.The object_reference
    field specifies a reference to a URI Object defined elsewhere in
    the document, via its id."""

    subclass = None
    superclass = None
    def __init__(self, object_reference=None):
        self.object_reference = _cast(None, object_reference)
        pass
    def factory(*args_, **kwargs_):
        if LinkReferenceType.subclass:
            return LinkReferenceType.subclass(*args_, **kwargs_)
        else:
            return LinkReferenceType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_object_reference(self): return self.object_reference
    def set_object_reference(self, object_reference): self.object_reference = object_reference
    def hasContent_(self):
        if (

            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='LinkReferenceType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='LinkReferenceType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='LinkReferenceType'):
        if self.object_reference is not None:

            lwrite(' object_reference=%s' % (quote_attrib(self.object_reference), ))
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='LinkReferenceType', fromsubclass_=False, pretty_print=True):
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('object_reference', node)
        if value is not None:

            self.object_reference = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class LinkReferenceType

class EmailMessageObjectType(cybox_common.ObjectPropertiesType):
    """The EmailMessageObjectType type is intended to characterize an
    individual email message."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Header=None, Email_Server=None, Raw_Body=None, Raw_Header=None, Attachments=None, Links=None):
        super(EmailMessageObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.Header = Header
        self.Email_Server = Email_Server
        self.Raw_Body = Raw_Body
        self.Raw_Header = Raw_Header
        self.Attachments = Attachments
        self.Links = Links
    def factory(*args_, **kwargs_):
        if EmailMessageObjectType.subclass:
            return EmailMessageObjectType.subclass(*args_, **kwargs_)
        else:
            return EmailMessageObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Header(self): return self.Header
    def set_Header(self, Header): self.Header = Header
    def get_Email_Server(self): return self.Email_Server
    def set_Email_Server(self, Email_Server): self.Email_Server = Email_Server
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Raw_Body(self): return self.Raw_Body
    def set_Raw_Body(self, Raw_Body): self.Raw_Body = Raw_Body
    def get_Raw_Header(self): return self.Raw_Header
    def set_Raw_Header(self, Raw_Header): self.Raw_Header = Raw_Header
    def get_Attachments(self): return self.Attachments
    def set_Attachments(self, Attachments): self.Attachments = Attachments
    def get_Links(self): return self.Links
    def set_Links(self, Links): self.Links = Links
    def hasContent_(self):
        if (
            self.Header is not None or
            self.Email_Server is not None or
            self.Raw_Body is not None or
            self.Raw_Header is not None or
            self.Attachments is not None or
            self.Links is not None or
            super(EmailMessageObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailMessageObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailMessageObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='EmailMessageObj:', name_='EmailMessageObjectType'):
        super(EmailMessageObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='EmailMessageObjectType')
    def exportChildren(self, lwrite, level, namespace_='EmailMessageObj:', name_='EmailMessageObjectType', fromsubclass_=False, pretty_print=True):
        super(EmailMessageObjectType, self).exportChildren(lwrite, level, 'EmailMessageObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Header is not None:
            self.Header.export(lwrite, level, 'EmailMessageObj:', name_='Header', pretty_print=pretty_print)
        if self.Email_Server is not None:
            self.Email_Server.export(lwrite, level, 'EmailMessageObj:', name_='Email_Server', pretty_print=pretty_print)
        if self.Raw_Body is not None:
            if self.Raw_Body.get_valueOf_() is not None:
                value = self.Raw_Body.get_valueOf_()
                if not value.startswith('<![CDATA['):
                    value = '<![CDATA[' + value + ']]>'
                    self.Raw_Body.set_valueOf_(value)
            self.Raw_Body.export(lwrite, level, 'EmailMessageObj:', name_='Raw_Body', pretty_print=pretty_print)
        if self.Raw_Header is not None:
            if self.Raw_Header.get_valueOf_() is not None:
                value = self.Raw_Header.get_valueOf_()
                if not value.startswith('<![CDATA['):
                    value = '<![CDATA[' + value + ']]>'
                    self.Raw_Header.set_valueOf_(value)
            self.Raw_Header.export(lwrite, level, 'EmailMessageObj:', name_='Raw_Header', pretty_print=pretty_print)
        if self.Attachments is not None:
            self.Attachments.export(lwrite, level, 'EmailMessageObj:', name_='Attachments', pretty_print=pretty_print)
        if self.Links is not None:
            self.Links.export(lwrite, level, 'EmailMessageObj:', name_='Links', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(EmailMessageObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Header':
            obj_ = EmailHeaderType.factory()
            obj_.build(child_)
            self.set_Header(obj_)
        elif nodeName_ == 'Email_Server':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Email_Server(obj_)
        elif nodeName_ == 'Raw_Body':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Raw_Body(obj_)
        elif nodeName_ == 'Raw_Header':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Raw_Header(obj_)
        elif nodeName_ == 'Attachments':
            obj_ = AttachmentsType.factory()
            obj_.build(child_)
            self.set_Attachments(obj_)
        elif nodeName_ == 'Links':
            obj_ = LinksType.factory()
            obj_.build(child_)
            self.set_Links(obj_)
        super(EmailMessageObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class EmailMessageObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'User_Agent': cybox_common.StringObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Dependencies': cybox_common.DependenciesType,
    'By': cybox_common.StringObjectPropertyType,
    'Segment_Hash': cybox_common.HashValueType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'SubDatum': cybox_common.MetadataType,
    'Sender': address_object.AddressObjectType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Subject': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'From': cybox_common.StringObjectPropertyType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'For': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Import': cybox_common.StringObjectPropertyType,
    'Raw_Header': cybox_common.StringObjectPropertyType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Raw_Body': cybox_common.StringObjectPropertyType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'X_Priority': cybox_common.PositiveIntegerObjectPropertyType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Contributors': cybox_common.PersonnelType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'X_Mailer': cybox_common.StringObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'In_Reply_To': cybox_common.StringObjectPropertyType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'MIME_Version': cybox_common.StringObjectPropertyType,
    'Boundary': cybox_common.StringObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Timestamp': cybox_common.DateTimeObjectPropertyType,
    'Message_ID': cybox_common.StringObjectPropertyType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Recipient': address_object.AddressObjectType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Errors_To': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Precedence': cybox_common.StringObjectPropertyType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'X_Originating_IP': address_object.AddressObjectType,
    'Email_Server': cybox_common.StringObjectPropertyType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'VLAN_Name': cybox_common.StringObjectPropertyType,
    'Address': address_object.AddressObjectType,
    'Reply_To': address_object.AddressObjectType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'With': cybox_common.StringObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Address_Value': cybox_common.StringObjectPropertyType,
    'VLAN_Num': cybox_common.IntegerObjectPropertyType,
    'Content_Type': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'ID': cybox_common.StringObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Email_Message'
        rootClass = EmailMessageObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Email_Message'
        rootClass = EmailMessageObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Email_Message'
        rootClass = EmailMessageObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Email_Message",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "EmailMessageObjectType",
    "AttachmentsType",
    "EmailHeaderType",
    "EmailRecipientsType",
    "LinksType",
    "EmailReceivedLineType",
    "EmailReceivedLineListType",
    "AttachmentReferenceType",
    "LinkReferenceType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.custom_object import Custom
from cybox.test.objects import ObjectTestCase


class TestCustom(ObjectTestCase, unittest.TestCase):
    object_type = "CustomObjectType"
    klass = Custom

    _full_dict = {
        'custom_name': u("SuperCustomizedObjectName"),
        'description': "Some description of the custom object",
        'custom_properties': [
            {'name': "Property #1", 'description': "A First Property", 'value': "12345"},
            {'name': "Property #2", 'description': "A Second Property", 'value': "true"},
        ],
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.network_packet_object as network_packet_binding
from cybox.common import (DataSegment, HexBinary, Integer, ObjectProperties,
        PositiveInteger, String, UnsignedInteger)
from cybox.objects.address_object import Address
from cybox.objects.port_object import Port


class TypeLength(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.TypeLengthType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    # TODO: choice
    length = fields.TypedField("Length", HexBinary)
    internet_layer_type = fields.TypedField("Internet_Layer_Type", String)


class EthernetHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.EthernetHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    destination_mac_addr = fields.TypedField("Destination_MAC_Addr", Address)
    source_mac_addr = fields.TypedField("Source_MAC_Addr", Address)
    type_or_length = fields.TypedField("Type_Or_Length", TypeLength)
    checksum = fields.TypedField("Checksum", HexBinary)


class EthernetInterface(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.EthernetInterfaceType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ethernet_header = fields.TypedField("Ethernet_Header", EthernetHeader)


class PhysicalInterface(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.PhysicalInterfaceType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ethernet = fields.TypedField("Ethernet", EthernetInterface)


class ARP(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ARPType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    hardware_addr_type = fields.TypedField("Hardware_Addr_Type", String)
    proto_addr_type = fields.TypedField("Proto_Addr_Type", String)
    hardware_addr_size = fields.TypedField("Hardware_Addr_Size", HexBinary)
    proto_addr_size = fields.TypedField("Proto_Addr_Size", HexBinary)
    op_type = fields.TypedField("Op_Type", String)
    sender_hardware_addr = fields.TypedField("Sender_Hardware_Addr", Address)
    sender_protocol_addr = fields.TypedField("Sender_Protocol_Addr", Address)
    recip_hardware_addr = fields.TypedField("Recip_Hardware_Addr", Address)
    recip_protocol_addr = fields.TypedField("Recip_Protocol_Addr", Address)


class _ICMPHeader(entities.Entity):
    """Abstract Type"""
    _binding = network_packet_binding
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    type_ = fields.TypedField("Type", HexBinary)
    code = fields.TypedField("Code", HexBinary)
    checksum = fields.TypedField("Checksum", HexBinary)


class ICMPv4Header(_ICMPHeader):
    _binding_class = network_packet_binding.ICMPv4HeaderType


class ICMPv6Header(_ICMPHeader):
    _binding_class = network_packet_binding.ICMPv6HeaderType


class NDPLinkAddr(entities.Entity):
    """Abstract Type"""
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NDPLinkAddrType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    length = fields.TypedField("Length", HexBinary)
    link_layer_mac_addr = fields.TypedField("Link_Layer_MAC_Addr", Address)


class RouterSolicitationOptions(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RouterSolicitationOptionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    src_link_addr = fields.TypedField("Src_Link_Addr", NDPLinkAddr)


class RouterSolicitation(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RouterSolicitationType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    options = fields.TypedField("Options", RouterSolicitationOptions,
                               multiple=True)


class NDPMTU(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NDPMTUType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    length = fields.TypedField("Length", Integer)
    mtu = fields.TypedField("MTU", Integer)


class Prefix(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.PrefixType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    ipv6_addr = fields.TypedField("IPv6_Addr", Address)
    ip_addr_prefix = fields.TypedField("IP_Addr_Prefix", Address)


class NDPPrefixInfo(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NDPPrefixInfoType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    link_flag = fields.TypedField("link_flag")
    addr_config_flag = fields.TypedField("addr_config_flag")
    length = fields.TypedField("Length", HexBinary)
    prefix_length = fields.TypedField("Prefix_Length", Integer)
    valid_lifetime = fields.TypedField("Valid_Lifetime", Integer)
    preferred_lifetime = fields.TypedField("Preferred_Lifetime", Integer)
    prefix = fields.TypedField("Prefix", Prefix)


class RouterAdvertisementOptions(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RouterAdvertisementOptionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    src_link_addr = fields.TypedField("Src_Link_Addr", NDPLinkAddr)
    mtu = fields.TypedField("MTU", NDPMTU)
    prefix_info = fields.TypedField("Prefix_Info", NDPPrefixInfo)


class RouterAdvertisement(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RouterAdvertisementType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    managed_address_config_flag = \
            fields.TypedField("managed_address_config_flag")
    other_config_flag = fields.TypedField("other_config_flag")
    cur_hop_limit = fields.TypedField("Cur_Hop_Limit", Integer)
    router_lifetime = fields.TypedField("Router_Lifetime", Integer)
    reachable_time = fields.TypedField("Reachable_Time", Integer)
    retrans_timer = fields.TypedField("Retrans_Timer", Integer)
    options = fields.TypedField("Options", RouterAdvertisementOptions)


class NeighborSolicitationOptions(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NeighborSolicitationOptionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    src_link_addr = fields.TypedField("Src_Link_Addr", NDPLinkAddr)


class NeighborSolicitation(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NeighborSolicitationType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    target_ipv6_addr = fields.TypedField("Target_IPv6_Addr", Address)
    options = fields.TypedField("Options", NeighborSolicitationOptions)


class NeighborOptions(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NeighborOptionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    target_link_addr = fields.TypedField("Target_Link_Addr", NDPLinkAddr)


class NeighborAdvertisement(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NeighborAdvertisementType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    router_flag = fields.TypedField("router_flag")
    solicited_flag = fields.TypedField("solicited_flag")
    override_flag = fields.TypedField("override_flag")
    target_ipv6_addr = fields.TypedField("Target_IPv6_Addr", Address)
    options = fields.TypedField("Options", NeighborOptions)


class NDPRedirectedHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NDPRedirectedHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    length = fields.TypedField("Length", HexBinary)
    ipheader_and_data = fields.TypedField("IPHeader_And_Data", HexBinary)


class RedirectOptions(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RedirectOptionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    target_link_addr = fields.TypedField("Target_Link_Addr", NDPLinkAddr)
    redirected_header = fields.TypedField("Redirected_Header",
                                          NDPRedirectedHeader)


class Redirect(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RedirectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    target_ipv6_addr = fields.TypedField("Target_IPv6_Addr", Address)
    dest_ipv6_addr = fields.TypedField("Dest_IPv6_Addr", Address)
    options = fields.TypedField("Options", RedirectOptions)


class NDP(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NDPType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    icmpv6_header = fields.TypedField("ICMPv6_Header", ICMPv6Header)

    #TODO: choice between these 5
    router_solicitation = fields.TypedField("Router_Solicitation",
                                           RouterSolicitation)
    router_advertisement = fields.TypedField("Router_Advertisement",
                                            RouterAdvertisement)
    neighbor_solicitation = fields.TypedField("Neighbor_Solicitation",
                                             NeighborSolicitation)
    neighbor_advertisement = fields.TypedField("Neighbor_Advertisement",
                                              NeighborAdvertisement)
    redirect = fields.TypedField("Redirect", Redirect)


class LogicalProtocol(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.LogicalProtocolType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    arp_rarp = fields.TypedField("ARP_RARP", ARP)
    ndp = fields.TypedField("NDP", NDP)


class LinkLayer(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.LinkLayerType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    physical_interface = fields.TypedField("Physical_Interface", PhysicalInterface)
    logical_protocols = fields.TypedField("Logical_Protocols", LogicalProtocol)


class IPv4Flags(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv4FlagsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    reserved = fields.TypedField("Reserved", Integer)
    do_not_fragment = fields.TypedField("Do_Not_Fragment", String)
    more_fragments = fields.TypedField("More_Fragments", String)

    def __init__(self):
        super(IPv4Flags, self).__init__()
        self.reserved = Integer(0)


class IPv4Option(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv4OptionType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    copy_flag = fields.TypedField("Copy_Flag", String)
    class_ = fields.TypedField("Class", String)
    option = fields.TypedField("Option", String)


class IPv4Header(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv4HeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ip_version = fields.TypedField("IP_Version", String)
    header_length = fields.TypedField("Header_Length", Integer)
    dscp = fields.TypedField("DSCP", HexBinary)
    ecn = fields.TypedField("ECN", HexBinary)
    total_length = fields.TypedField("Total_Length", HexBinary)
    identification = fields.TypedField("Identification", PositiveInteger)
    flags = fields.TypedField("Flags", IPv4Flags)
    fragment_offset = fields.TypedField("Fragment_Offset", HexBinary)
    ttl = fields.TypedField("TTL", HexBinary)
    protocol = fields.TypedField("Protocol", String)
    checksum = fields.TypedField("Checksum", HexBinary)
    src_ipv4_addr = fields.TypedField("Src_IPv4_Addr", Address)
    dest_ipv4_addr = fields.TypedField("Dest_IPv4_Addr", Address)
    option = fields.TypedField("Option", IPv4Option, multiple=True)


class IPv4Packet(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv4PacketType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ipv4_header = fields.TypedField("IPv4_Header", IPv4Header)
    data = fields.TypedField("Data", HexBinary)


class FragmentationRequired(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.FragmentationRequiredType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    fragmentation_required = fields.TypedField("Fragmentation_Required")
    next_hop_mtu = fields.TypedField("Next_Hop_MTU", HexBinary)


class ICMPv4DestinationUnreachable(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4DestinationUnreachableType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    destination_network_unreachable = \
            fields.TypedField("Destination_Network_Unreachable")
    destination_host_unreachable = \
            fields.TypedField("Destination_Host_Unreachable")
    destination_protocol_unreachable = \
            fields.TypedField("Destination_Protocol_Unreachable")
    destination_port_unreachable = \
            fields.TypedField("Destination_Port_Unreachable")
    fragmentation_required = fields.TypedField("Fragmentation_Required",
                                             FragmentationRequired)
    source_route_failed = fields.TypedField("Source_Route_Failed")
    destination_network_unknown = \
            fields.TypedField("Destination_Network_Unknown")
    destination_host_unknown = fields.TypedField("Destination_Host_Unknown")
    source_host_isolated = fields.TypedField("Source_Host_Isolated")
    network_administratively_prohibited = \
            fields.TypedField("Network_Administratively_Prohibited")
    host_administratively_prohibited = \
            fields.TypedField("Host_Administratively_Prohibited")
    network_unreachable_for_tos = \
            fields.TypedField("Network_Unreachable_For_TOS")
    host_unreachable_for_tos = \
            fields.TypedField("Host_Unreachable_For_TOS")
    communication_administratively_prohibited = \
            fields.TypedField("Communication_Administratively_Prohibited")
    host_precedence_violation = fields.TypedField("Host_Precedence_Violation")
    precedence_cutoff_in_effect = \
            fields.TypedField("Precedence_Cutoff_In_Effect")


class ICMPv4SourceQuench(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4SourceQuenchType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    source_quench = fields.TypedField("Source_Quench")


class ICMPv4RedirectMessage(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4RedirectMessageType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    # TODO: choice of first 4
    network_redirect = fields.TypedField("Network_Redirect")
    host_redirect = fields.TypedField("Host_Redirect")
    tos_network_redirect = fields.TypedField("ToS_Network_Redirect")
    tos_host_redirect = fields.TypedField("ToS_Host_Redirect")

    ip_address = fields.TypedField("IP_Address", Address)


class ICMPv4TimeExceeded(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4TimeExceededType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    # TODO: choice
    ttl_exceeded_in_transit = fields.TypedField("TTL_Exceeded_In_Transit")
    frag_reassembly_time_exceeded = \
            fields.TypedField("Frag_Reassembly_Time_Exceeded")


class ICMPv4ErrorMessageContent(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4ErrorMessageContentType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ip_header = fields.TypedField("IP_Header", IPv4Header)
    first_eight_bytes = fields.TypedField("First_Eight_Bytes", HexBinary)


class ICMPv4ErrorMessage(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4ErrorMessageType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    destination_unreachable = fields.TypedField("Destination_Unreachable",
                                                ICMPv4DestinationUnreachable)
    source_quench = fields.TypedField("Source_Quench", ICMPv4SourceQuench)
    redirect_message = fields.TypedField("Redirect_Message",
                                        ICMPv4RedirectMessage)
    time_exceeded = fields.TypedField("Time_Exceeded", ICMPv4TimeExceeded)
    error_msg_content = fields.TypedField("Error_Msg_Content",
                                        ICMPv4ErrorMessageContent)


class _ICMPEchoReply(entities.Entity):
    _binding = network_packet_binding
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    echo_reply = fields.TypedField("Echo_Reply")
    data = fields.TypedField("Data", HexBinary)


class _ICMPEchoRequest(entities.Entity):
    _binding = network_packet_binding
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    echo_request = fields.TypedField("Echo_Request")
    data = fields.TypedField("Data", HexBinary)


class ICMPv4EchoReply(_ICMPEchoReply):
    _binding_class = network_packet_binding.ICMPv4EchoReplyType


class ICMPv4EchoRequest(_ICMPEchoRequest):
    _binding_class = network_packet_binding.ICMPv4EchoRequestType


class ICMPv6EchoReply(_ICMPEchoReply):
    _binding_class = network_packet_binding.ICMPv6EchoReplyType


class ICMPv6EchoRequest(_ICMPEchoRequest):
    _binding_class = network_packet_binding.ICMPv6EchoRequestType


class ICMPv4TimestampRequest(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4TimestampRequestType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    timestamp = fields.TypedField("Timestamp")
    originate_timestamp = fields.TypedField("Originate_Timestamp",
                                           UnsignedInteger)


class ICMPv4TimestampReply(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4TimestampReplyType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    timestamp_reply = fields.TypedField("Timestamp_Reply")
    originate_timestamp = fields.TypedField("Originate_Timestamp",
                                           UnsignedInteger)
    receive_timestamp = fields.TypedField("Receive_Timestamp",
                                         UnsignedInteger)
    transmit_timestamp = fields.TypedField("Transmit_Timestamp",
                                          UnsignedInteger)


class ICMPv4AddressMaskRequest(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4AddressMaskRequestType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    address_mask_request = fields.TypedField("Address_Mask_Request")
    address_mask = fields.TypedField("Address_Mask", Address)


class ICMPv4AddressMaskReply(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4AddressMaskReplyType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    address_mask_reply = fields.TypedField("Address_Mask_Reply")
    address_mask = fields.TypedField("Address_Mask", Address)


class _ICMPInfoMessageContent(entities.Entity):
    _binding = network_packet_binding
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    identifier = fields.TypedField("Identifier", HexBinary)
    sequence_number = fields.TypedField("Sequence_Number", HexBinary)


class ICMPv4InfoMessageContent(_ICMPInfoMessageContent):
    _binding_class = network_packet_binding.ICMPv4InfoMessageContentType


class ICMPv6InfoMessageContent(_ICMPInfoMessageContent):
    _binding_class = network_packet_binding.ICMPv6InfoMessageContentType


class ICMPv4InfoMessage(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4InfoMessageType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice of first 6
    echo_reply = fields.TypedField("Echo_Reply", ICMPv4EchoReply)
    echo_request = fields.TypedField("Echo_Request", ICMPv4EchoRequest)
    timestamp_request = fields.TypedField("Timestamp_Request",
                                         ICMPv4TimestampRequest)
    timestamp_reply = fields.TypedField("Timestamp_Reply", ICMPv4TimestampReply)
    address_mask_request = fields.TypedField("Address_Mask_Request",
                                            ICMPv4AddressMaskRequest)
    address_mask_reply = fields.TypedField("Address_Mask_Reply",
                                          ICMPv4AddressMaskReply)

    info_msg_content = fields.TypedField("Info_Msg_Content",
                                        ICMPv4InfoMessageContent)


class ICMPv4Traceroute(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4TracerouteType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    outbound_packet_forward_success = \
            fields.TypedField("Outbound_Packet_Forward_Success")
    outbound_packet_no_route = fields.TypedField("Outbound_Packet_no_Route")
    identifier = fields.TypedField("Identifier", HexBinary)
    outbound_hop_count = fields.TypedField("Outbound_Hop_Count", HexBinary)
    return_hop_count = fields.TypedField("Return_Hop_Count", HexBinary)
    output_link_speed = fields.TypedField("Output_Link_Speed", HexBinary)
    output_link_mtu = fields.TypedField("Output_Link_MTU", HexBinary)


class ICMPv4Packet(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv4PacketType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    icmpv4_header = fields.TypedField("ICMPv4_Header", ICMPv4Header)

    # TODO: choice between these 3
    error_msg = fields.TypedField("Error_Msg", ICMPv4ErrorMessage)
    info_msg = fields.TypedField("Info_Msg", ICMPv4InfoMessage)
    traceroute = fields.TypedField("Traceroute", ICMPv4Traceroute)


class IPv6Header(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv6HeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ip_version = fields.TypedField("IP_Version", String)
    traffic_class = fields.TypedField("Traffic_Class", HexBinary)
    flow_label = fields.TypedField("Flow_Label", HexBinary)
    payload_length = fields.TypedField("Payload_Length", HexBinary)
    next_header = fields.TypedField("Next_Header", String)
    ttl = fields.TypedField("TTL", HexBinary)
    src_ipv6_addr = fields.TypedField("Src_IPv6_Addr", Address)
    dest_ipv6_addr = fields.TypedField("Dest_IPv6_Addr", Address)


class IPv6Option(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv6OptionType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    do_not_recogn_action = fields.TypedField("Do_Not_Recogn_Action", String)
    packet_change = fields.TypedField("Packet_Change", String)
    option_byte = fields.TypedField("Option_Byte", HexBinary)


class Pad1(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.Pad1Type
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    octet = fields.TypedField("Octet", HexBinary)


class PadN(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.PadNType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    octet = fields.TypedField("Octet", HexBinary)
    option_data_length = fields.TypedField("Option_Data_Length", Integer)
    option_data = fields.TypedField("Option_Data", Integer)


class OptionData(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.OptionDataType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    option_type = fields.TypedField("Option_Type", IPv6Option)
    option_data_len = fields.TypedField("Option_Data_Len", HexBinary)

    #TODO: choice
    pad1 = fields.TypedField("Pad1", Pad1)
    padn = fields.TypedField("PadN", PadN)


class _IPv6ExtHeader(entities.Entity):
    """Shared by a couple types"""
    _binding = network_packet_binding
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    next_header = fields.TypedField("Next_Header", String)
    header_ext_len = fields.TypedField("Header_Ext_Len", HexBinary)
    option_data = fields.TypedField("Option_Data", OptionData, multiple=True)


class HopByHopOptions(_IPv6ExtHeader):
    _binding_class = network_packet_binding.HopByHopOptionsType


class Routing(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.RoutingType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    next_header = fields.TypedField("Next_Header", String)
    header_ext_len = fields.TypedField("Header_Ext_Len", Integer)
    routing_type = fields.TypedField("Routing_Type", HexBinary)
    segments_left = fields.TypedField("Segments_Left", Integer)
    type_specific_data = fields.TypedField("Type_Specific_Data", String)


class FragmentHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.FragmentHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    next_header = fields.TypedField("Next_Header", HexBinary)
    fragment_offset = fields.TypedField("Fragment_Offset", HexBinary)
    m_flag = fields.TypedField("M_Flag", String)
    identification = fields.TypedField("Identification", HexBinary)


class Fragment(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.FragmentType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    fragment_header = fields.TypedField("Fragment_Header", FragmentHeader)
    fragment = fields.TypedField("Fragment", HexBinary)


class DestinationOptions(_IPv6ExtHeader):
    _binding_class = network_packet_binding.DestinationOptionsType


class AuthenticationHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.AuthenticationHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    next_header = fields.TypedField("Next_Header", String)
    header_ext_len = fields.TypedField("Header_Ext_Len", HexBinary)
    security_parameters_index = fields.TypedField("Security_Parameters_Index",
                                                 HexBinary)
    sequence_number = fields.TypedField("Sequence_Number", HexBinary)
    authentication_data = fields.TypedField("Authentication_Data", HexBinary)


class EncapsulatingSecurityPayload(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.EncapsulatingSecurityPayloadType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    security_parameters_index = fields.TypedField("Security_Parameters_Index",
                                                 HexBinary)
    sequence_number = fields.TypedField("Sequence_Number", HexBinary)
    payload_data = fields.TypedField("Payload_Data", HexBinary)
    padding = fields.TypedField("Padding", HexBinary)
    padding_len = fields.TypedField("Padding_Len", HexBinary)
    next_header = fields.TypedField("Next_Header", String)
    authentication_data = fields.TypedField("Authentication_Data", HexBinary)


class IPv6ExtHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv6ExtHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    hop_by_hop_options = fields.TypedField("Hop_by_Hop_Options", HopByHopOptions)
    routing = fields.TypedField("Routing", Routing)
    fragment = fields.TypedField("Fragment", Fragment)
    destination_options = fields.TypedField("Destination_Options", DestinationOptions, multiple=True)
    authentication_header = fields.TypedField("Authentication_Header", AuthenticationHeader)
    encapsulating_security_payload = fields.TypedField("Encapsulating_Security_Payload", EncapsulatingSecurityPayload)


class IPv6Packet(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.IPv6PacketType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ipv6_header = fields.TypedField("IPv6_Header", IPv6Header)
    data = fields.TypedField("Data", HexBinary)
    ext_headers = fields.TypedField("Ext_Headers", IPv6ExtHeader, multiple=True)


class ICMPv6DestinationUnreachable(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6DestinationUnreachableType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    no_route = fields.TypedField("No_Route")
    comm_prohibited = fields.TypedField("Comm_Prohibited")
    beyond_scope = fields.TypedField("Beyond_Scope")
    address_unreachable = fields.TypedField("Address_Unreachable")
    port_unreachable = fields.TypedField("Port_Unreachable")
    src_addr_failed_policy = fields.TypedField("Src_Addr_Failed_Policy")
    reject_route = fields.TypedField("Reject_Route")


class ICMPv6PacketTooBig(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6PacketTooBigType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    packet_too_big = fields.TypedField("Packet_Too_Big")
    mtu = fields.TypedField("MTU", HexBinary)


class ICMPv6TimeExceeded(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6TimeExceededType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    hop_limit_exceeded = fields.TypedField("Hop_Limit_Exceeded")
    fragment_reassem_time_exceeded = \
            fields.TypedField("Fragment_Reassem_Time_Exceeded")


class ICMPv6ParameterProblem(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6ParameterProblemType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice of first 3
    erroneous_header_field = fields.TypedField("Erroneous_Header_Field")
    unrecognized_next_header_type = \
            fields.TypedField("Unrecognized_Next_Header_Type")
    unrecognized_ipv6_option = fields.TypedField("Unrecognized_IPv6_Option")

    pointer = fields.TypedField("Pointer", HexBinary)


class ICMPv6ErrorMessage(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6ErrorMessageType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    destination_unreachable = fields.TypedField("Destination_Unreachable",
                                                ICMPv6DestinationUnreachable)
    packet_too_big = fields.TypedField("Packet_Too_Big", ICMPv6PacketTooBig)
    time_exceeded = fields.TypedField("Time_Exceeded", ICMPv6TimeExceeded)
    parameter_problem = fields.TypedField("Parameter_Problem",
                                         ICMPv6ParameterProblem)
    invoking_packet = fields.TypedField("Invoking_Packet", HexBinary)


class ICMPv6InfoMessage(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6InfoMessageType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice of first 2
    echo_request = fields.TypedField("Echo_Request", ICMPv6EchoRequest)
    echo_reply = fields.TypedField("Echo_Reply", ICMPv6EchoReply)

    info_msg_content = fields.TypedField("Info_Msg_Content",
                                        ICMPv6InfoMessageContent)


class ICMPv6Packet(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.ICMPv6PacketType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    icmpv6_header = fields.TypedField("ICMPv6_Header", ICMPv6Header)
    error_msg = fields.TypedField("Error_Msg", ICMPv6ErrorMessage)
    info_msg = fields.TypedField("Info_Msg", ICMPv6InfoMessage)


class InternetLayer(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.InternetLayerType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    #TODO: choice
    ipv4 = fields.TypedField("IPv4", IPv4Packet)
    icmpv4 = fields.TypedField("ICMPv4", ICMPv4Packet)
    ipv6 = fields.TypedField("IPv6", IPv6Packet)
    icmpv6 = fields.TypedField("ICMPv6", ICMPv6Packet)


class TCPFlags(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.TCPFlagsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    ns = fields.TypedField("ns")
    cwr = fields.TypedField("cwr")
    ece = fields.TypedField("ece")
    urg = fields.TypedField("urg")
    ack = fields.TypedField("ack")
    psh = fields.TypedField("psh")
    rst = fields.TypedField("rst")
    syn = fields.TypedField("syn")
    fin = fields.TypedField("fin")


class TCPHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.TCPHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    src_port = fields.TypedField("Src_Port", Port)
    dest_port = fields.TypedField("Dest_Port", Port)
    seq_num = fields.TypedField("Seq_Num", HexBinary)
    ack_num = fields.TypedField("ACK_Num", HexBinary)
    data_offset = fields.TypedField("Data_Offset", HexBinary)
    reserved = fields.TypedField("Reserved", Integer)
    tcp_flags = fields.TypedField("TCP_Flags", TCPFlags)
    window = fields.TypedField("Window", HexBinary)
    checksum = fields.TypedField("Checksum", HexBinary)
    urg_ptr = fields.TypedField("Urg_Ptr", HexBinary)


class TCP(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.TCPType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    tcp_header = fields.TypedField("TCP_Header", TCPHeader)
    options = fields.TypedField("Options", HexBinary)
    data = fields.TypedField("Data", DataSegment)


class UDPHeader(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.UDPHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    srcport = fields.TypedField("SrcPort", Port)
    destport = fields.TypedField("DestPort", Port)
    length = fields.TypedField("Length", Integer)
    checksum = fields.TypedField("Checksum", HexBinary)


class UDP(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.UDPType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    udp_header = fields.TypedField("UDP_Header", UDPHeader)
    data = fields.TypedField("Data", DataSegment)


class TransportLayer(entities.Entity):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.TransportLayerType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"

    tcp = fields.TypedField("TCP", TCP)
    udp = fields.TypedField("UDP", UDP)


class NetworkPacket(ObjectProperties):
    _binding = network_packet_binding
    _binding_class = network_packet_binding.NetworkPacketObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/network-packet-2"
    _XSI_NS = "PacketObj"
    _XSI_TYPE = "NetworkPacketObjectType"

    link_layer = fields.TypedField("Link_Layer", LinkLayer)
    internet_layer = fields.TypedField("Internet_Layer", InternetLayer)
    transport_layer = fields.TypedField("Transport_Layer", TransportLayer)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_system_restore_object import WinSystemRestore

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestWinSystemRestore(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsSystemRestoreObjectType"
    klass = WinSystemRestore

    _full_dict = {
        'restore_point_description': u("restore desc"),
        'restore_point_full_path': u("restore path"),
        'acl_change_username': u("username"),
        'restore_point_name': u("pont name"),
        'restore_point_type': u("point type"),
        'backup_file_name': u("backup name"),
        'acl_change_sid': u("an SID"),
        'changelog_entry_flags': u("entry flags"),
        'changelog_entry_sequence_number': 1234,
        # TODO: add 'created'
        'file_attributes': u("RWX"),
        'new_file_name': u("New name"),
        'original_file_name': u("original file name"),
        'original_short_file_name': u("org fname"),
        'process_name': u("A process name"),
        'change_event': u("some event"),
        'changelog_entry_type': u("entry type"),
        'registry_hive_list': [
            u("hive 1"),
            u("hive 2")
        ],
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields
from mixbox.vendor import six

import cybox.bindings.cybox_common as common_binding


@six.python_2_unicode_compatible
class StructuredText(entities.Entity):
    _binding = common_binding
    _binding_class = _binding.StructuredTextType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    value = fields.TypedField("valueOf_", key_name="value")
    structuring_format = fields.TypedField("structuring_format")

    def __init__(self, value=None):
        super(StructuredText, self).__init__()
        self.value = value

    def to_dict(self):
        # Shortcut if structuring_format is not defined.
        if self.is_plain():
            return self.value
        return super(StructuredText, self).to_dict()

    def is_plain(self):
        """Whether this can be represented as a string rather than a dictionary

        Subclasses can override this to include their custom fields in this
        check:

            return (super(..., self).is_plain() and self.other_field is None)
        """
        return (self.structuring_format is None)

    def __str__(self):
        return six.text_type(self.value)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox.namespaces import lookup_name

from cybox.core import Observable
from cybox.objects import _OBJ_META, get_class_for_object_type
import cybox.test


class ObjectTestCase(cybox.test.EntityTestCase):
    """A base class for testing all subclasses of ObjectProperties.

    Each subclass of ObjectTestCase should subclass both unittest.TestCase
    and ObjectTestCase, and defined two class-level fields:
    - klass: the ObjectProperties subclass being tested
    - object_type: The name prefix used in the XML Schema bindings for the
      object.
    """
    def test_round_trip_dict(self):
        # We don't want to run this test on this (abstract) class
        if type(self) is not ObjectTestCase:
            super(ObjectTestCase, self).test_round_trip_dict()

    def test_type_exists(self):
        # Verify that the correct class has been added to the metadata lists.

        # Skip this base class
        if type(self) is ObjectTestCase:
            return

        t = self.__class__.object_type

        expected_class = get_class_for_object_type(t)
        actual_class = self.__class__.klass

        self.assertEqual(expected_class, actual_class)

        expected_namespace = expected_class._XSI_NS
        namespace = _OBJ_META.lookup_object(t).namespace
        actual_namespace = lookup_name(namespace)
        self.assertEqual(expected_namespace, actual_namespace)

        self.assertEqual(expected_class._XSI_TYPE, t)

    def test_round_trip_observable(self):
        # Wrap the object in an observable before trying a round_trip test.

        # Don't run this test on the base class
        if type(self) is ObjectTestCase:
            return

        obj = self.klass.from_dict(self._full_dict)
        observable = Observable(obj)
        observable2 = cybox.test.round_trip(observable, output=True)
        self.maxDiff = None
        self.assertEqual(observable.to_dict(), observable2.to_dict())

    def test_object_reference(self, obj_dict=None):
        klass = self.__class__.klass

        if not obj_dict:
            obj_dict = {}

        obj_dict['object_reference'] = "some:object-reference-1"
        obj_dict['xsi:type'] = klass._XSI_TYPE

        obj_dict2 = cybox.test.round_trip_dict(klass, obj_dict)
        self.assertEqual(obj_dict, obj_dict2)



#!/usr/bin/env python

# Copyright (c) 2015 - The MITRE Corporation
# For license information, see the LICENSE.txt file

from os.path import abspath, dirname, join


from setuptools import setup, find_packages

BASE_DIR = dirname(abspath(__file__))
VERSION_FILE = join(BASE_DIR, 'cybox', 'version.py')

def get_version():
    with open(VERSION_FILE) as f:
        for line in f.readlines():
            if line.startswith("__version__"):
                version = line.split()[-1].strip('"')
                return version
        raise AttributeError("Package does not have a __version__")


with open('README.rst') as f:
    readme = f.read()

install_requires = [
    'lxml>=2.2.3',
    'mixbox>=0.0.13',
    'python-dateutil',
]

try:
    import importlib
except ImportError:
    install_requires.append('importlib')

extras_require = {
    'docs': [
        'Sphinx==1.3.1',
        'sphinx_rtd_theme==0.1.8',
    ],
    'test': [
        "nose==1.3.0",
        "tox==1.6.1"
    ],
}

setup(
    name="cybox211",
    version=get_version(),
    author="CybOX Project, MITRE Corporation",
    author_email="cybox@mitre.org",
    description="A Python library for parsing and generating CybOX content.",
    long_description=readme,
    url="http://cybox.mitre.org",
    packages=find_packages(),
    install_requires=install_requires,
    extras_require=extras_require,
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ]
)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
import collections

from mixbox import entities
from mixbox import fields
from mixbox import idgen

from cybox import Unicode
import cybox.bindings.cybox_core as core_binding
from cybox.common import MeasureSource, ObjectProperties, StructuredText
from cybox.core import Object, Event


def validate_operator(instance, value):
    allowed = ObservableComposition.OPERATORS

    if value in allowed:
        return

    error = "Operator must be one of {allowed}. Received '{value}'."
    raise ValueError(error.format(**locals()))


def validate_object(instance, value):
    if not value:
        return
    elif not isinstance(value, Object):
        raise TypeError('value must be an Object')
    elif instance.event:
        raise ValueError("Observable already has an Event.")
    elif instance.observable_composition:
        raise ValueError("Observable already has an ObservableComposition.")


def validate_event(instance, value):
    if not value:
        return
    elif not isinstance(value, Event):
        raise TypeError("value must be an Event")
    elif instance.object_:
        raise ValueError("Observable already has an Object.")
    elif instance.observable_composition:
        raise ValueError("Observable already has an ObservableComposition.")


def validate_observable_composition(instance, value):
    if not value:
        return
    elif not isinstance(value, ObservableComposition):
        raise TypeError('value must be an ObservableComposition')
    elif instance.object_:
        raise ValueError("Observable already has an Object.")
    elif instance.event:
        raise ValueError("Observable already has an Event.")


class Keywords(entities.EntityList):
    _binding = core_binding
    _binding_class = core_binding.KeywordsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'
    keyword = fields.TypedField("Keyword", Unicode, multiple=True)

class Observable(entities.Entity):
    """A single Observable.
    """
    _binding = core_binding
    _binding_class = _binding.ObservableType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    id_ = fields.IdField("id")
    idref = fields.IdrefField("idref")
    title = fields.TypedField("Title")
    description = fields.TypedField("Description", StructuredText)
    object_ = fields.TypedField("Object", Object, preset_hook=validate_object)  # TODO: Add preset hook
    event = fields.TypedField("Event", Event, preset_hook=validate_event)
    observable_composition = fields.TypedField("Observable_Composition", type_="cybox.core.ObservableComposition", preset_hook=validate_observable_composition)
    sighting_count = fields.TypedField("sighting_count")
    observable_source = fields.TypedField("Observable_Source", MeasureSource, multiple=True)
    keywords = fields.TypedField("Keywords", Keywords)
    pattern_fidelity = fields.TypedField("Pattern_Fidelity", type_="cybox.core.PatternFidelity")

    def __init__(self, item=None, id_=None, idref=None, title=None, description=None):
        """Create an Observable out of 'item'.

        `item` can be any of:
        - an Object
        - an Event
        - an ObservableComposition
        - any subclass of ObjectProperties.

        In the first three cases, the appropriate property of the Observable
        will be set. In the last cases, an Object will be built automatically
        to ensure the correct hierarchy is created.
        """
        super(Observable, self).__init__()

        self.id_ = id_ or idgen.create_id(prefix="Observable")
        self.idref = idref
        self.title = title
        self.description = description
        self.keywords = Keywords()

        if item is None:
            return
        elif isinstance(item, Object):
            self.object_ = item
        elif isinstance(item, ObservableComposition):
            self.observable_composition = item
        elif isinstance(item, Event):
            self.event = item
        elif isinstance(item, ObjectProperties):
            if item.parent:
                self.object_ = item.parent
            else:
                self.object_ = Object(item)
        else:
            msg = ("item must be an Object, Event, ObservableComposition, or "
                   "subclass of ObjectProperties. Received an %s" % type(item))
            raise TypeError(msg)

    def add_keyword(self, value):
        self.keywords.append(value)


class Observables(entities.EntityList):
    """The root CybOX Observables object.

    Pools are not currently supported.
    """
    _binding = core_binding
    _binding_class = _binding.ObservablesType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    observable_package_source = fields.TypedField("Observable_Package_Source", MeasureSource)
    observables = fields.TypedField("Observable", Observable, multiple=True, key_name="observables")

    def __init__(self, observables=None):
        super(Observables, self).__init__(observables)
        # Assume major_verion and minor_version are immutable for now
        self._major_version = 2
        self._minor_version = 1
        self._update_version = 0

    def add(self, observable):
        if not observable:
            return
        if not isinstance(observable, Observable):
            observable = Observable(observable)
        self.observables.append(observable)

    def to_obj(self, ns_info=None):
        observables_obj = super(Observables, self).to_obj(ns_info=ns_info)
        observables_obj.cybox_major_version = self._major_version
        observables_obj.cybox_minor_version = self._minor_version
        observables_obj.cybox_update_version = self._update_version
        return observables_obj

    def to_dict(self):
        observables_dict = super(Observables, self).to_dict()
        observables_dict['major_version'] = self._major_version
        observables_dict['minor_version'] = self._minor_version
        observables_dict['update_version'] = self._update_version
        return observables_dict


class ObservableComposition(entities.EntityList):
    """The ObservableCompositionType entity defines a logical compositions of
    CybOX Observables. The combinatorial behavior is derived from the operator
    property."""

    _binding = core_binding
    _binding_class = _binding.ObservableCompositionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    OPERATOR_AND = 'AND'
    OPERATOR_OR = 'OR'
    OPERATORS = (OPERATOR_AND, OPERATOR_OR)

    operator = fields.TypedField("operator", preset_hook=validate_operator)
    observables = fields.TypedField("Observable", Observable, multiple=True, key_name="observables")

    def __init__(self, operator='AND', observables=None):
        super(ObservableComposition, self).__init__(observables)
        self.operator = operator

    def add(self, observable):
        if not observable:
            raise ValueError("'observable' must not be None")
        self.append(observable)




# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.core import Observables
from cybox.objects.link_object import Link
from cybox.objects.uri_object import URI
from cybox.test.objects import ObjectTestCase


class TestLink(ObjectTestCase, unittest.TestCase):
    object_type = "LinkObjectType"
    klass = Link

    _full_dict = {
        'value': u("http://www.example.com"),
        'type': URI.TYPE_URL,
        'url_label': u("Click Here!"),
        'xsi:type': object_type,
    }

    # https://github.com/CybOXProject/python-cybox/issues/202
    def test_correct_namespace_output(self):
        link = Link()
        link.value = u("https://www.example.com")

        xml = Observables(link).to_xml()
        self.assertTrue(b"cybox:Properties" in xml)
        self.assertTrue(b"LinkObj:Properties" not in xml)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.address_object import Address
from cybox.objects.socket_address_object import SocketAddress
import cybox.test
from cybox.test.objects import ObjectTestCase


class TestSocketAddress(ObjectTestCase, unittest.TestCase):
    object_type = "SocketAddressObjectType"
    klass = SocketAddress

    _full_dict = {
        # Normally only one out of IP_Address and Hostname is allowed,
        # but we don't enforce that, so we can test both here.
        'ip_address': {
            'category': Address.CAT_IPV4,
            'address_value': u("192.168.1.1"),
            'xsi:type': "AddressObjectType"
        },
        'hostname': {
            'is_domain_name': True,
            'hostname_value': u("www.example.com"),
            'xsi:type': "HostnameObjectType"
        },
        'port': {
            'port_value': 80,
            'layer4_protocol': u("TCP"),
            'xsi:type': "PortObjectType"
        },
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox import entities
from mixbox import fields

from cybox.objects.address_object import Address


class DummyEntity(entities.Entity):

    var1 = fields.TypedField("Var1")
    var2 = fields.TypedField("Var2", comparable=False)


class ComparatorTest(unittest.TestCase):

    def setUp(self):
        self.a1 = Address("someone@example.org")
        self.a2 = Address("someone@example.org")
        self.a3 = self.a1
        self.a4 = Address("someone_else@example.org")

    def test_equal(self):
        # Equal but not the same object
        self.assertEqual(self.a1, self.a2)
        self.assertTrue(self.a1 is not self.a2)

    def test_same(self):
        # The same object, by two different names, should compare as equal.
        self.assertEqual(self.a1, self.a3)
        self.assertTrue(self.a1 is self.a3)

    def test_not_equal(self):
        self.assertNotEqual(self.a1, self.a4)

    def test_other_properties(self):
        # We can set other properties (besides TypedFields) and they still
        # compare equal.
        self.a1.blah = True
        self.a2.blah = False
        self.assertEqual(self.a1, self.a2)

        # But if we set one of the TypedFields, they are no longer equal.
        self.a1.is_source = True
        self.assertNotEqual(self.a1, self.a2)

        self.a2.is_source = True
        self.assertEqual(self.a1, self.a2)

    def test_non_comparable_fields(self):
        ent1 = DummyEntity()
        ent2 = DummyEntity()

        ent1.var1 = "blah"
        ent2.var1 = "blah"

        # var2 is not comparable, so this should not affect the equality check.
        ent1.var2 = "blah2"
        ent2.var2 = "blah3"
        self.assertEqual(ent1, ent2)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.dns_record_object as dns_record_binding
from cybox.common import (Integer, HexBinary, ObjectProperties, String,
        StructuredText, DateTime)
from cybox.objects.address_object import Address
from cybox.objects.uri_object import URI


class DNSRecord(ObjectProperties):
    _binding = dns_record_binding
    _binding_class = dns_record_binding.DNSRecordObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/dns-record-2'
    _XSI_NS = "DNSRecordObj"
    _XSI_TYPE = "DNSRecordObjectType"

    description = fields.TypedField("Description", StructuredText)
    domain_name = fields.TypedField("Domain_Name", URI)
    queried_date = fields.TypedField("Queried_Date", DateTime)
    ip_address = fields.TypedField("IP_Address", Address)
    address_class = fields.TypedField("Address_Class", String)
    entry_type = fields.TypedField("Entry_Type", String)
    record_name = fields.TypedField("Record_Name", String)
    record_type = fields.TypedField("Record_Type", String)
    ttl = fields.TypedField("TTL", Integer)
    flags = fields.TypedField("Flags", HexBinary)
    data_length = fields.TypedField("Data_Length", Integer)
    record_data = fields.TypedField("Record_Data")
